<?php
/**
 * Perforce Swarm
 *
 * @copyright   2013-2024 Perforce Software. All rights reserved.
 * @license     Please see LICENSE.txt in top-level readme folder of this distribution.
 * @version     2022.2/2545213
 */

namespace Application;

use Application\Config\ConfigException;
use Application\Config\ConfigManager as Cm;
use Application\Config\IConfigDefinition as IDef;
use Application\Config\Services;
use Application\Permissions\Exception\Exception as PermissionsException;
use Application\Permissions\IPermissions;
use Interop\Container\ContainerInterface;
use Application\Factory\InvokableService;

/**
 * Class Checker. Abstract checker for subclassing
 * @package Application
 */
abstract class Checker implements InvokableService
{
    const CHECKERS     = 'checkers';
    const NAME         = 'name';
    const VALUE        = 'value';
    const OPTIONS      = 'options';
    const RETURN_VALUE = 'returnValue';

    protected $services;

    /**
     * @inheritDoc
     */
    public function __construct(ContainerInterface $services, array $options = null)
    {
        $this->services = $services;
    }

    /**
     * Carry out a check
     * @param string            $check      name of the check
     * @param array|null        $options    optional data to assist the check
     */
    abstract protected function check(string $check, array $options = null);

    /**
     * Gets the configuration setting specified and if it is true performs an admin check for the current user. If the
     * admin check is performed and fails the exception is caught so that false can be returned.
     * @throws ConfigException
     * @return bool
     *  - config setting true, admin true: return true
     *  - config setting true, admin false: return false
     *  - config setting false: return true
     */
    protected function checkAdminForConfigSetting(string $configSetting): bool
    {
        $valid       = true;
        $config      = $this->services->get(IDef::CONFIG);
        $configValue = Cm::getValue($config, $configSetting, false);
        if ($configValue) {
            $configCheck = $this->services->get(Services::CONFIG_CHECK);
            try {
                $configCheck->check(IPermissions::ADMIN_CHECKER);
            } catch (PermissionsException $e) {
                $valid = false;
            }
        }
        return $valid;
    }
}
